# ====================================================================
# Copyright (c) 2006-2010 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from datetime import datetime
from icu import *

class TestUDate(TestCase):

    def testConvertNaiveDatetime(self):

        def formatNaiveDatetime():
            # Make a naive datetime
            dt = datetime(2006, 4, 18, 5, 12)
            # ... and format it
            return DateFormat.createTimeInstance(DateFormat.SHORT).format(dt)

        before = formatNaiveDatetime()
        # Change ICU's default timezone
        TimeZone.setDefault(ICUtzinfo.getInstance('US/Eastern').timezone)
        after = formatNaiveDatetime()

        self.assertTrue(before == after)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2021 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *

class TestRuleBasedNumberFormat(TestCase):

    def testSpellout(self):

        formatter = RuleBasedNumberFormat(
            URBNFRuleSetTag.SPELLOUT, Locale("fr_FR"))

        text = formatter.format(80, "%spellout-ordinal-masculine")
        self.assertEqual(text, u'quatre-vingtième')
        text = formatter.format(76, '%spellout-ordinal-feminine-plural')
        self.assertEqual(text, u'soixante-seizièmes')
                
        formatter = RuleBasedNumberFormat(
            URBNFRuleSetTag.SPELLOUT, Locale("fr_CH"))

        text = formatter.format(70)
        self.assertEqual(text, u'septante')
        text = formatter.format(76, '%spellout-ordinal-feminine-plural')
        self.assertEqual(text, u'septante-sixièmes')
                

if __name__ == "__main__":
    main()



# ====================================================================
# Copyright (c) 2024-2024 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================

import sys, os, six

from unittest import TestCase, main
from icu import *


class TestUnicodeSet(TestCase):

    def testIterators(self):

        a = UnicodeSet("[a-d{abc}{de}]")
        self.assertEqual(['a', 'b', 'c', 'd', 'abc', 'de'], list(a))
        self.assertEqual(['a', 'b', 'c', 'd'], list(a.codePoints()))
        self.assertEqual(['abc', 'de'], list(a.strings()))

        a = UnicodeSet("[abcçカ🚴{}{abc}{de}]")
        self.assertEqual([('a', 'c'), ('ç', 'ç'), ('カ', 'カ'), ('🚴', '🚴')],
                         list(a.ranges()))


if __name__ == "__main__":
    main()



# ====================================================================
# Copyright (c) 2007-2010 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os, six

from unittest import TestCase, main
from icu import *

# We use u'stuff'.encode('encoding') instead of b'stuff' to represent
# bytes using literals. This is to ensure 2to3 doesn't turn things
# into unicode literals, but also to keep compatibility with Python
# pre 2.6, which didn't know about the b prefix to literals.

class TestCharset(TestCase):

    def testDetect(self):

        detector = CharsetDetector()
        detector.setText(u'foo'.encode('ascii'))

        match = detector.detect()
        self.assertTrue(match.getName() == 'UTF-8')

    def testDetectAll(self):

        detector = CharsetDetector(u'foo'.encode('ascii'))

        matches = detector.detectAll()
        self.assertTrue(matches[0].getName() == 'UTF-8')

    def testDeclared(self):

        bytes = u'beaut\xe9 probable'.encode('iso-8859-1')
        detector = CharsetDetector(bytes, 'iso-8859-1')

        self.assertTrue("ISO-8859-1" in (m.getName()
                                         for m in detector.detectAll()))

    def testUnicode(self):

        bytes = u'beaut\xe9 probable'.encode('iso-8859-1')
        ustring = six.text_type(CharsetDetector(bytes).detect())

        self.assertTrue(ustring.encode('iso-8859-1') == bytes)

    def testEnableInputFilter(self):

        detector = CharsetDetector()
        detector.enableInputFilter(True)

        self.assertTrue(detector.isInputFilterEnabled())

        detector.enableInputFilter(False)

        self.assertFalse(detector.isInputFilterEnabled())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2009-2010 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *


class TestNormalizer2(TestCase):

    def assertNorm(self, normalizer2, expected, source):
        self.assertEqual(expected, normalizer2.normalize(source))
        a = UnicodeString()
        normalizer2.normalize(source, a)
        self.assertEqual(UnicodeString(expected), a)

    def testNormalize(self):

        try:
            from icu import Normalizer2
        except ImportError:
            return

        self.assertNorm(Normalizer2.getInstance(None, "nfkc_cf",
                                                UNormalizationMode2.COMPOSE),
                        u'hi there', "Hi There")
        self.assertNorm(Normalizer2.getNFCInstance(),
                        u"äßáＷ", u"äßa\u0301Ｗ")
        self.assertNorm(Normalizer2.getNFDInstance(),
                        u"a\u0308ßa\u0301Ｗ", u"äßa\u0301Ｗ")
        self.assertNorm(Normalizer2.getNFKCInstance(),
                        u"äßáW", u"äßa\u0301Ｗ")
        self.assertNorm(Normalizer2.getNFKDInstance(),
                        u"a\u0308ßa\u0301W", u"äßa\u0301Ｗ")
        self.assertNorm(Normalizer2.getNFKCCasefoldInstance(),
                        u"ässáw", u"äßa\u0301Ｗ")


if __name__ == "__main__":
    main()



# ====================================================================
# Copyright (c) 2005-2018 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os, six

from unittest import TestCase, main
from icu import *

class TestCollator(TestCase):

    def filePath(self, name):

        module = sys.modules[TestCollator.__module__].__file__
        return os.path.join(os.path.dirname(module), name)

    def assertIsInstance(self, obj, cls):
        if hasattr(TestCase, 'assertIsInstance'):
            TestCase.assertIsInstance(self, obj, cls)
        else:
            self.assertTrue(isinstance(obj, cls),
                            u'%s is not an instance of %s' % (obj, cls))

    def testSort(self):

        collator = Collator.createInstance(Locale.getFrance())
        input = open(self.filePath('noms.txt'), 'rb')
        names = [six.text_type(n.strip(), 'utf-8') for n in input.readlines()]
        input.close()
        ecole = names[0]

        names.sort()
        self.assertTrue(names[-1] is ecole)

        if (sys.version_info >= (3,)):
            names.sort(key=collator.getSortKey)
        else:
            names.sort(collator.compare)
        self.assertTrue(names[2] is ecole)

    def testCreateInstancePolymorph(self):

        collator = Collator.createInstance(Locale("epo")) # Esperanto
        self.assertIsInstance(collator, RuleBasedCollator)
        rules = collator.getRules()

    def testGetSortKey(self):

        # Do not test sort key byte sequences directly:
        # They are unstable, that is, likely to change
        # with every UCA/CLDR/ICU release.
        # Instead, test that compare() is consistent with
        # comparing the equivalent sort keys.
        collator = Collator.createInstance(Locale.getJapanese())
        collator.setAttribute(UCollAttribute.NORMALIZATION_MODE,
                              UCollAttributeValue.ON)
        collator.setAttribute(UCollAttribute.ALTERNATE_HANDLING,
                              UCollAttributeValue.SHIFTED)
        collator.setAttribute(UCollAttribute.STRENGTH,
                              UCollAttributeValue.TERTIARY)

        # In Japanese, the following characters should be different
        # only on quaternary level.
        hira_ge = u'\u3052'  # Hiragana letter Ge
        kana_ge = u'\u30B2'  # Katakana letter Ge
        self.assertEqual(0, collator.compare(hira_ge, kana_ge))
        hira_ge_key = collator.getSortKey(hira_ge)
        kana_ge_key = collator.getSortKey(kana_ge)
        self.assertEqual(hira_ge_key, kana_ge_key)

        collator.setAttribute(UCollAttribute.STRENGTH,
                              UCollAttributeValue.QUATERNARY)
        self.assertEqual(-1, collator.compare(hira_ge, kana_ge))
        hira_ge_key = collator.getSortKey(hira_ge)
        kana_ge_key = collator.getSortKey(kana_ge)
        self.assertTrue(hira_ge_key < kana_ge_key)

    def setupCollator(self, collator):

        collator.setAttribute(UCollAttribute.NORMALIZATION_MODE,
                              UCollAttributeValue.ON)
        collator.setAttribute(UCollAttribute.CASE_FIRST,
                              UCollAttributeValue.UPPER_FIRST)
        collator.setAttribute(UCollAttribute.ALTERNATE_HANDLING,
                              UCollAttributeValue.SHIFTED)
        collator.setAttribute(UCollAttribute.STRENGTH,
                              UCollAttributeValue.QUATERNARY)
        collator.setAttribute(UCollAttribute.HIRAGANA_QUATERNARY_MODE,
                              UCollAttributeValue.ON)

    def LoadCollatorFromRules(self):

        rules = u"&z<\u00e6 &h<ch"
        collator = RuleBasedCollator(rules)
        self.setupCollator(collator)

        return collator

    def LoadCollatorFromBinaryBuffer(self, bin):

        root = Collator.createInstance(Locale.getRoot())
        collator = RuleBasedCollator(bin, root)
        self.setupCollator(collator)

        return collator

    def testCollatorLoading(self):

        if ICU_VERSION >= '4.6':
            collator = self.LoadCollatorFromRules()
            s = u'hchz\u00e6'

            key0 = collator.getSortKey(s)
            bin = collator.cloneBinary()

            collator = self.LoadCollatorFromBinaryBuffer(bin)
            key1 = collator.getSortKey(s)

            self.assertTrue(key0 == key1)

    def testAlphabeticIndex(self):

        if ICU_VERSION >= '4.8':
            index = AlphabeticIndex(Locale.getItaly())

            index.addRecord("foo", "bar")
            index.addRecord("topo", "lino")

            def allData(index):
                for ((label, type)) in index:
                    while index.nextRecord():
                        yield (label, type, index.recordData)

            self.assertTrue(list(allData(index)) == [
                ('F', 0, 'bar'), ('T', 0, 'lino')])

            if ICU_VERSION >= '51.0':
                self.assertTrue(len(index.buildImmutableIndex()) == 28)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2023 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================

import sys, os, six

from unittest import TestCase, main
from icu import *

class TestRegexMatcher(TestCase):

    def testReset(self):

        s = "Thuɔŋjäŋ"
        us = UnicodeString(s)
        matcher = RegexMatcher("t.+", URegexpFlag.CASE_INSENSITIVE)
        matcher.reset(us)
        self.assertTrue(matcher.matches())

        matcher.reset(s)
        self.assertTrue(matcher.matches())

        s = "དཔལ་དགོན་ཀླུ་གྲུབ།"
        us = UnicodeString(s)
        matcher = RegexMatcher('\\X')
        matcher.reset(us)
        results = []
        finds = ['ད', 'པ', 'ལ', '་', 'ད', 'གོ', 'ན', '་', 'ཀླུ', '་', 'གྲུ', 'བ', '།']
        while matcher.find():
            results.append(matcher.group())
        self.assertEqual(finds, results)

        matcher.reset(s)
        results = []
        while matcher.find():
            results.append(matcher.group())
        self.assertEqual(finds, results)

        
if __name__ == "__main__":
    main()



# ====================================================================
# Copyright (c) 2005-2010 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *

class Tests(TestCase):

    def testSortOfHighAndLowCodepoints(self):
        collator = Collator.createInstance(Locale.getFrance())
        self.assertEqual(collator.compare(u'\U00023119', u'\u62d5'),
                         1,
                         "Failed to collate low character before high",
                         )
        self.assertEqual(collator.compare(u'\U00023119'*10000,
                                          u'\u62d5'*10000),
                         1,
                         "Failed to collate low character before high",
                         )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2005-2010 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *


class TestMessageFormat(TestCase):

    def testFormatMessage(self):

        x = 'x'
        f = Formattable(UnicodeString(x))

        text = MessageFormat.formatMessage("This is a string: {0}.", [f])
        self.assertTrue(text == "This is a string: x.")

    def testFormat(self):

        x = 'x'
        f = Formattable(UnicodeString(x))
        msgFormat = MessageFormat("This is a string: {0}")

        text = msgFormat.format([f], FieldPosition())
        self.assertTrue(text == "This is a string: x")

        if (sys.version_info < (3,) and 'PyPy' not in sys.version):
            # Reduced features of % operator might be intentional
            f = UnicodeString(x)
            text = msgFormat % f
            self.assertTrue(text == "This is a string: x")

    def testFormatAppend(self):

        x = 'x'
        f = Formattable(UnicodeString(x))
        msgFormat = MessageFormat("This is a string: {0}")

        text = msgFormat.format([f], UnicodeString("x"), FieldPosition())
        self.assertTrue(text == "xThis is a string: x")

    def testFormats(self):

        orig = TimeZone.createDefault().getID()
        tzid = 'America/Indiana/Vevay'        
        msgFormat = MessageFormat("{0,time,short} {0,time,z}")
        formats = msgFormat.getFormats()

        formats[0].setTimeZone(TimeZone.createTimeZone(tzid))
        self.assertTrue(msgFormat.getFormats()[0].getTimeZone().getID() == orig)

        msgFormat.setFormats(formats)
        self.assertTrue(msgFormat.getFormats()[0].getTimeZone().getID() == tzid)

    def testSelectFormat(self):

        try:
            from icu import SelectFormat
        except ImportError:
            return

        format = u"{0} est {1, select, female {allée} other {allé}} à Paris."
        msgFormat = MessageFormat(format, Locale("fr"))
        args = [Formattable("Kirti"), Formattable("female")]

        self.assertTrue(msgFormat.format(args) == u"Kirti est allée à Paris.")

    def testNamedArgs(self):

        if ICU_VERSION >= '4.0':
            msg = \
                '{num_emails_to_send, plural, ' \
                '=0 {unused plural form} ' \
                '=1 {One email will be sent.} ' \
                'other {# emails will be sent.}}'

            messageFormat = MessageFormat(msg, Locale("en"))

            name0 = "num_emails_to_send"
            arg0 = Formattable(5)

            result = messageFormat.format([name0], [arg0])
            self.assertTrue(result == u'5 emails will be sent.')


if __name__ == "__main__":
    main()



# ====================================================================
# Copyright (c) 2021-2021 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os, six, datetime

from unittest import TestCase, main
from icu import *


class TestTimeZone(TestCase):

    def testBasic(self):

        tz = TimeZone.createTimeZone("America/Los_Angeles")
        self.assertTrue(isinstance(tz, BasicTimeZone))
        self.assertEqual(str(tz), "America/Los_Angeles")

    if ICU_VERSION >= '4.8' and ICU_VERSION < '76.0':
        def testGetRegion(self):

            self.assertEqual('US', TimeZone.getRegion("America/Los_Angeles"))
            self.assertEqual('001', TimeZone.getRegion("CET"))

    if ICU_VERSION >= '76.0':
        def testGetRegion(self):

            self.assertEqual('US', TimeZone.getRegion("America/Los_Angeles"))
            self.assertEqual('BE', TimeZone.getRegion("CET"))

    def testGetRules(self):

        tz = TimeZone.createTimeZone("America/Los_Angeles")
        rules = tz.getTimeZoneRules()
        self.assertTrue(len(rules) > 0)
        self.assertTrue(isinstance(rules[0], InitialTimeZoneRule))
        for rule in rules[1:]:
          self.assertTrue(
              isinstance(rule, (TimeArrayTimeZoneRule, AnnualTimeZoneRule)))

    def testVTimeZone(self):

        vtz = VTimeZone.createVTimeZoneByID("Pacific/Fiji")
        data = vtz.write()

        vtz.createVTimeZone(data)
        self.assertEqual(vtz.write(), data)

        data = vtz.writeSimple(
            datetime.datetime(2021, 4, 10, 12, 43, 33, 411400))
        self.assertTrue(data.startswith(
            "BEGIN:VTIMEZONE\r\nTZID:Pacific/Fiji\r\n"))

        if ICU_VERSION >= '4.6':
            tz = TimeZone.createTimeZone("America/Los_Angeles")
            vtz = VTimeZone.createVTimeZoneFromBasicTimeZone(tz)
            data = vtz.write()
            vtz.createVTimeZone(data)
            self.assertEqual(vtz.write(), data)

    def testTransition(self):

        tz = TimeZone.createTimeZone("Pacific/Fiji")
        pytz = ICUtzinfo.getInstance("Pacific/Fiji")

        tzr = tz.getNextTransition(
            datetime.datetime(2020, 4, 10, 12, 43, 33, tzinfo=pytz))

        tzr_time = tzr.getTime()
        self.assertEqual(datetime.datetime.fromtimestamp(tzr_time, pytz),
                         datetime.datetime(2020, 12, 20, 2, 0, tzinfo=pytz))

        pytz = ICUtzinfo.getInstance("America/Los_Angeles")
        self.assertEqual(datetime.datetime.fromtimestamp(tzr_time, pytz),
                         datetime.datetime(2020, 12, 19, 6, 0, tzinfo=pytz))


class TestTimeZoneNames(TestCase):

    def testGetAvailableMetaZoneIDs(self):
        if ICU_VERSION >= '50.0':
            tzn = TimeZoneNames.createInstance(Locale.getFrance())
            ids = list(tzn.getAvailableMetaZoneIDs("America/Chicago"))
            self.assertEqual(['America_Central'], ids)
            ids = list(tzn.getAvailableMetaZoneIDs("Asia/Calcutta"))
            self.assertEqual(['India'], ids)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2024 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from icu import *

class TestIntPropertyMap(TestCase):

    def setUp(self):
        if ICU_VERSION < '63.0':
            self.skipTest(ICU_VERSION)

    def testGet(self):

        eastAsianWidthMap = Char.getIntPropertyMap(UProperty.EAST_ASIAN_WIDTH)
        self.assertEqual(0, eastAsianWidthMap.get(ord('क')))  # Neutral
        self.assertEqual(1, eastAsianWidthMap.get(ord('○')))  # Ambiguous
        self.assertEqual(2, eastAsianWidthMap.get(ord('ｶ')))  # Halfwidth
        self.assertEqual(3, eastAsianWidthMap.get(ord('ａ')))  # Fullwidth
        self.assertEqual(4, eastAsianWidthMap.get(ord('a')))  # Narrow
        self.assertEqual(5, eastAsianWidthMap.get(ord('😄')))  # Wide

    def testGetRange(self):
        eastAsianWidthMap = Char.getIntPropertyMap(UProperty.EAST_ASIAN_WIDTH)
        self.assertEqual(42124, eastAsianWidthMap.getRange(ord('\u4E00')))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2021-2021 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *

class TestMessagePattern(TestCase):

    def testMessagePattern(self):
        text = 'Test this {PLACEHOLDER}'
        pattern = MessagePattern(text)
        self.assertEqual(pattern.countParts(), 5)

        part = pattern.getPart(2)
        self.assertEqual(text[part.getIndex():part.getLimit()], "PLACEHOLDER")
        self.assertEqual(part.getType(), UMessagePatternPartType.ARG_NAME)
        self.assertEqual(pattern.getPartType(2), UMessagePatternPartType.ARG_NAME)
        self.assertEqual(pattern.getSubstring(part), "PLACEHOLDER")
        self.assertEqual(str(part), '([11:22] type=8, arg=0, val=0)')
        self.assertEqual(repr(part), '<MessagePattern_Part: ([11:22] type=8, arg=0, val=0)>')

    def testEscapingRelated(self):
        text = "This pattern can't be '{SIMPLE}'"
        pattern = MessagePattern(UMessagePatternApostropheMode.DOUBLE_OPTIONAL)
        pattern.parse(text)
        self.assertEqual(str(pattern), text)
        self.assertEqual(pattern.autoQuoteApostropheDeep(),
                         "This pattern can''t be '{SIMPLE}'")

    def testComplexPattern(self):
        text = """{GENDER, select,
                    male {{NUMBER, plural,
                      one {He wrote 1 ICU message.}
                      other {He wrote # ICU messages.}}}
                    female {{NUMBER, plural,
                      one {She wrote 1 ICU message.}
                      other {She wrote # ICU messages.}}}
                    other {{NUMBER, plural,
                      one {They wrote 1 ICU message.}
                      other {They wrote # ICU messages.}}}}"""
        pattern = MessagePattern(text)
        types = set([pattern.getPartType(i)
                     for i in range(0, pattern.countParts())])
        expected = {
            UMessagePatternPartType.MSG_START,
            UMessagePatternPartType.MSG_LIMIT,
            UMessagePatternPartType.REPLACE_NUMBER,
            UMessagePatternPartType.ARG_START,
            UMessagePatternPartType.ARG_LIMIT,
            UMessagePatternPartType.ARG_NAME,
            UMessagePatternPartType.ARG_SELECTOR,
        }
        self.assertEqual(types, expected)

if __name__ == "__main__":
    main()




# ====================================================================
# Copyright (c) 2004-2010 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from icu import UnicodeString, BreakIterator, Locale


def printTextRange(iterator, start, end):

    s = iterator.getText().getText()
    print("%2d %2d  %s|%s|%s" %(start, end, s[:start], s[start:end], s[end:]))

#def printTextRange(iterator, start, end):
#
#    u = iterator.getText().getText(UnicodeString())
#    print "%2d %2d  %s|%s|%s" %(start, end,
#                                UnicodeString(u, 0, start),
#                                UnicodeString(u, start, end-start),
#                                UnicodeString(u, end))


def printEachForward(boundary):

    start = boundary.first()
    for end in boundary:
        printTextRange(boundary, start, end)
        start = end


# Print each element in reverse order:
def printEachBackward(boundary):

    end = boundary.last()
    while True:
        start = boundary.previous()
        if start == BreakIterator.DONE:
            break
        printTextRange(boundary, start, end)
        end = start


# Print the first element
def printFirst(boundary):

    start = boundary.first()
    end = boundary.next()
    printTextRange(boundary, start, end)


# Print the last element
def printLast(boundary):

    end = boundary.last()
    start = boundary.previous()
    if start != BreakIterator.DONE:
        printTextRange(boundary, start, end)


# Print the element at a specified position
def printAt(boundary, pos):

    end = boundary.following(pos)
    start = boundary.previous()
    printTextRange(boundary, start, end)


def main():

    print("ICU Break Iterator Sample Program")
    print("C++ Break Iteration in Python")
    
    stringToExamine = u"Aaa bbb ccc. Ddd eee fff."
    print("Examining: ", stringToExamine)

    # print each sentence in forward and reverse order
    boundary = BreakIterator.createSentenceInstance(Locale.getUS())
    boundary.setText(stringToExamine)

    print()
    print("Sentence Boundaries... ")
    print("----- forward: -----------")
    printEachForward(boundary)
    print("----- backward: ----------")
    printEachBackward(boundary)

    # print each word in order
    print()
    print("Word Boundaries...")
    boundary = BreakIterator.createWordInstance(Locale.getUS())
    boundary.setText(stringToExamine)
    print("----- forward: -----------")
    printEachForward(boundary)
    # print first element
    print("----- first: -------------")
    printFirst(boundary)
    # print last element
    print("----- last: --------------")
    printLast(boundary)
    # print word at charpos 10
    print("----- at pos 10: ---------")
    printAt(boundary, 10)

    print()
    print("End C++ Break Iteration in Python")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2019 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================

import sys, os, six

from unittest import TestCase, main
from icu import *


class _Replaceable(object):

    def __init__(self, buffer):
        self.buffer = buffer

    def getLength(self):
        return len(self.buffer)

    def getCharAt(self, offset):
        return self.buffer[offset]

    def getChar32At(self, offset):
        return self.buffer[offset]

    def extractBetween(self, start, limit):
        return self.buffer[start:limit]

    def handleReplaceBetween(self, start, limit, text):
        self.buffer = ''.join((self.buffer[:start], text, self.buffer[limit:]))

    def copy(self, start, limit, dest):
        self.buffer = ''.join((self.buffer[:dest], self.buffer[start:limit],
                               self.buffer[dest:]))

    def hasMetaData(self):
        return False


class TestPythonReplaceable(TestCase):

    def setUp(self):
        if ICU_VERSION < '55.0':
            self.skipTest(ICU_VERSION)

    def testAccess(self):

        _rep = _Replaceable('abcd')    # the python implementation
        rep = PythonReplaceable(_rep)  # the wrapper to pass to ICU

        self.assertEqual(4, rep.length())
        self.assertEqual(ord('b'), rep.charAt(1))
        self.assertEqual(ord('c'), rep.char32At(2))
        self.assertEqual('bc', rep.extractBetween(1, 3))

        rep.handleReplaceBetween(1, 3, 'pqrs')
        self.assertEqual('apqrsd', _rep.buffer)

    def testTransliterate(self):

        trans = Transliterator.createInstance('Accents-Any')
        inverse = trans.createInverse()

        string = u'\xe9v\xe9nement'
        if ICU_VERSION < '4.0':
            result = u"e<'>ve<'>nement"
        else:
            result = u"e\u2190'\u2192ve\u2190'\u2192nement"

        _rep = _Replaceable(string)    # the python implementation
        rep = PythonReplaceable(_rep)  # the wrapper to pass to ICU

        trans.transliterate(rep)
        self.assertEqual(result, _rep.buffer)

        inverse.transliterate(rep)
        self.assertEqual(string, _rep.buffer)


if __name__ == "__main__":
    main()



# ====================================================================
# Copyright (c) 2007-2013 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#
# This is a python translation of ICU's LocaleDataTest.java

import sys, os, six

from unittest import TestCase, main
from icu import *

NOT_FOUND_ERR = 'The requested resource cannot be found, error code: 2'

def print_output(string):
  pass

class ExemplarGroup:
    def __init__(_self, s, scriptCodes):
        _self.set = s
        _self.scs = scriptCodes
    def __hash__(_self):
        hash = 0
        for i in range(len(_self.scs) if len(_self.scs) < 4 else 4):
            hash = (hash << 8) + _self.scs[i]
        return hash
    def __eq__(_self, other):
        return (_self.set == other.set) and (_self.scs == other.scs)


class TestLocaleData(TestCase):
    def setUp(self):
        self.availableLocales = Locale.getAvailableLocales()

    def testPapersize(self):
        for locale in self.availableLocales.keys():
            papersize = LocaleData(locale).getPaperSize()
            #language = Locale(locale).getLanguage()
            country = Locale(locale).getCountry()

            if (['BZ','CA','CL','CO','CR','GT','MX','NI','PA','PH','PR','SV','US','VE'].count(country) > 0):
                self.assertTrue(papersize == (279, 216))
            elif country:
                self.assertTrue(papersize == (297, 210))

    def testMeasurementSystem(self):
        for locale in self.availableLocales.keys():
            measurementSystem = LocaleData(locale).getMeasurementSystem()
            #language = Locale(locale).getLanguage()
            country = Locale(locale).getCountry()

            # 0 means SI, 1 means US, 2 mean UK
            if (country in ['LR', 'US']):
                self.assertTrue(measurementSystem == 1)
            elif country in ['GB']:
                self.assertTrue(measurementSystem in [0, 2])
            elif country in ['MM']:
                self.assertTrue(measurementSystem in [1, 2])
            elif country:
                self.assertTrue(measurementSystem == 0)

    def testExemplarSet(self):
        testedExemplars = set()
        equalCount = 0

        for locale in self.availableLocales:
            scriptCodes = Script.getCode(locale)
            exemplarSets = []
            for k in range(2):
                # it seems USET_CASE_INSENSITIVE = 2
                option = 0 if k == 0 else 2
                # standard = 0, auxiliary = 1, index = 2, count = 3
                exemplarSet = LocaleData(locale).getExemplarSet(option, 0)
                exemplarSets.append(exemplarSet)
                exGrp = ExemplarGroup(exemplarSet, scriptCodes)
                if (not exGrp in testedExemplars):
                    testedExemplars.add(exGrp)
                    sets = []
                    for j in range(len(scriptCodes)):
                        sets.append(UnicodeSet('[:' + Script(scriptCodes[j]).getShortName() + ':]'))
                    existsInScript = False
                    iter = UnicodeSetIterator(exemplarSet)
                    while (not existsInScript and iter.nextRange()):
                        if (not iter.isString()):
                            for j in range(len(sets)):
                                if sets[j].contains(iter.getCodepoint(), iter.getCodepointEnd()):
                                    existsInScript = True
                                    break
                        else:
                            for j in range(len(sets)):
                                if sets[j].contains(iter.getString()):
                                    existsInScript = True
                                    break
                    if existsInScript == False:
                        print_output("ExemplarSet containment failed for locale : "+ locale)
            print_output(locale + " exemplar " + repr(six.text_type(exemplarSets[0])))
            print_output(locale + " exemplar(case-folded) " + repr(six.text_type(exemplarSets[1])))
            self.assertTrue(locale + " case-folded is a superset", exemplarSets[1].containsAll(exemplarSets[0]))
            if (exemplarSets[1] == exemplarSets[0]):
                ++equalCount
        self.assertTrue("case-folded is sometimes a strict superset, and sometimes equal",\
                        equalCount > 0 and equalCount < len(self.availableLocales))

    def testExemplarSet2(self):
        testedExemplars = set()
        equalCount = 0

        for locale in self.availableLocales:
            ld = LocaleData(locale)
            scriptCodes = Script.getCode(locale)
            exemplarSets = []
            for k in range(2): # for casing option in (normal, uncased)
                # it seems USET_CASE_INSENSITIVE = 2
                option = 0 if k == 0 else 2
                for h in range(2):
                    esType = 0 if h == 0 else 1
                    # standard = 0, auxiliary = 1, index = 2, count = 3
                    exemplarSet = LocaleData(locale).getExemplarSet(option, esType)
                    exemplarSets.append(exemplarSet)
                    exGrp = ExemplarGroup(exemplarSet, scriptCodes)
                    if (not exGrp in testedExemplars):
                        testedExemplars.add(exGrp)
                        sets = []
                        for j in range(len(scriptCodes)):
                            sets.append(UnicodeSet('[:' + Script(scriptCodes[j]).getShortName() + ':]'))
                            existsInScript = False
                            iter = UnicodeSetIterator(exemplarSet)
                            while (not existsInScript and iter.nextRange()):
                                if (not iter.isString()):
                                    for j in range(len(sets)):
                                        if sets[j].contains(iter.getCodepoint(), iter.getCodepointEnd()):
                                            existsInScript = True
                                            break
                                else:
                                    for j in range(len(sets)):
                                        if sets[j].contains(iter.getString()):
                                            existsInScript = True
                                            break
                            if existsInScript == False and h == 0:
                                print_output("ExemplarSet containment failed for locale,option,type : " \
                                      + locale + "," + str(option) + "," + str(esType))
            print_output(locale + " exemplar(ES_STANDARD)" + repr(six.text_type(exemplarSets[0])))
            print_output(locale + " exemplar(ES_AUXILIARY)" + repr(six.text_type(exemplarSets[1])))
            print_output(locale + " exemplar(case-folded,ES_STANDARD)" + repr(six.text_type(exemplarSets[2])))
            print_output(locale + " exemplar(case-folded,ES_AUXILIARY)" + repr(six.text_type(exemplarSets[3])))
            self.assertTrue(locale + " case-folded is a superset", exemplarSets[2].containsAll(exemplarSets[0]))
            self.assertTrue(locale + " case-folder is a superset", exemplarSets[3].containsAll(exemplarSets[1]))
            if (exemplarSets[2] == exemplarSets[0]):
                ++equalCount
            if (exemplarSets[3] == exemplarSets[1]):
                ++equalCount
        self.assertTrue("case-folded is sometimes a strict superset, and sometimes equal",\
                        equalCount > 0 and equalCount < len(self.availableLocales) * 2)

    def testCoverage(self):
        ld = LocaleData(Locale().getName())
        t = ld.getNoSubstitute()
        ld.setNoSubstitute(t)
        self.assertEqual(t, ld.getNoSubstitute())
        for i in range(4):
            print_output(repr(ld.getDelimiter(i)))

    def testLocaleDisplayPattern(self):
        ld = LocaleData(Locale().getName())
        print_output("Default locale LocaleDisplayPattern:" + ld.getLocaleDisplayPattern());
        print_output("Default locale LocaleSeparator:" + ld.getLocaleSeparator());
        for locale in self.availableLocales:
            ld = LocaleData(locale)
            try:
                print_output(locale + " LocaleDisplayPattern:" + repr(ld.getLocaleDisplayPattern()))
                getLocaleDisplayPattern = True
            except ICUError as e:  # resource not found
                getLocaleDisplayPattern = str(e) == NOT_FOUND_ERR
            except:
                getLocaleDisplayPattern = False
            self.assertTrue(getLocaleDisplayPattern)
            try:
                print_output(locale + " LocaleSeparator:" + repr(ld.getLocaleSeparator()))
                getLocaleSeparator = True
            except ICUError as e:  # resource not found
                getLocaleSeparator = str(e) == NOT_FOUND_ERR
            except:
                getLocaleSeparator = False
            self.assertTrue(getLocaleSeparator)


if __name__ == "__main__":
    main()



# test package

import sys

if __name__ == "__main__":
    if sys.version_info >= (2, 7):
        import os, unittest

        def load_tests(loader, standard_tests, pattern):
            if pattern is None:
                pattern = 'test*.py'
            this_dir = os.path.dirname(__file__)
            package_tests = loader.discover(start_dir=this_dir, pattern=pattern)
            standard_tests.addTests(package_tests)
            return standard_tests

        unittest.main()
    else:
        print >>sys.stderr, 'Unit test discovery requires Python 2.7 or later'
        try:
            from pkg_resources import require
            with_setuptools = require('setuptools')[0].parsed_version
            print >>sys.stderr, "Use: 'python setup.py test' instead"
        except ImportError:
            pass



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2019 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================

import sys, os, six

from unittest import TestCase, main, SkipTest
from icu import *

class TestLocaleMatcher(TestCase):

    def setUp(self):
        if ICU_VERSION < '65.0':
            self.skipTest(ICU_VERSION)

    def testSetSupportedLocales(self):

        locales = (Locale.getFrance(), Locale.getGermany())
        matcher = LocaleMatcher.Builder().setSupportedLocales(locales).build()

        self.assertEqual(Locale.getFrance(),
                         matcher.getBestMatch(Locale.getItaly()))
        self.assertEqual(Locale.getGermany(),
                         matcher.getBestMatch(Locale('de-AT')))

    def testGetBestMatch(self):

        locales = (Locale.getFrance(), Locale.getGermany())
        matcher = LocaleMatcher.Builder().setSupportedLocales(locales).build()

        self.assertEqual(Locale.getGermany(),
                         matcher.getBestMatch((Locale('de-AT'), Locale('fr'))))

    def testGetBestMatchResult(self):

        locales = (Locale.getFrance(), Locale.getGermany())
        matcher = LocaleMatcher.Builder().setSupportedLocales(locales).build()
        result = matcher.getBestMatchResult((Locale('de-AT'), Locale('fr')))

        self.assertEqual(Locale('de-AT'), result.getDesiredLocale())
        self.assertEqual(Locale.getGermany(), result.getSupportedLocale())

    def testAcceptLanguage(self):

        locale, status = LocaleMatcher.acceptLanguage(
            ('fr-CH', 'fr', 'en', 'de', '*'), ('de-AT', 'fr-CA'))

        self.assertEqual('fr_CA', locale)
        self.assertEqual(UAcceptResult.FALLBACK, status)

        locale, status = LocaleMatcher.acceptLanguage(
            ('fr-CH', 'fr', 'en', 'de', '*'),
            list(Locale.getAvailableLocales().keys()))

        self.assertEqual('fr_CH', locale)
        self.assertEqual(UAcceptResult.VALID, status)

        locale, status = LocaleMatcher.acceptLanguage(
            ('fr-CH', 'fr', 'de'), ('en-CA',))
        self.assertEqual(None, locale)
        self.assertEqual(UAcceptResult.FAILED, status)

    def testAcceptLanguageFromHTTP(self):

        locale, status = LocaleMatcher.acceptLanguageFromHTTP(
            'fr-CH, fr;q=0.9, en;q=0.8, de;q=0.7, *;q=0.5', ('de-AT', 'fr-CA'))

        self.assertEqual('fr_CA', locale)
        self.assertEqual(UAcceptResult.FALLBACK, status)

        locale, status = LocaleMatcher.acceptLanguageFromHTTP(
            'fr-CH, fr;q=0.9, en;q=0.8, de;q=0.7, *;q=0.5',
            list(Locale.getAvailableLocales().keys()))

        self.assertEqual('fr_CH', locale)
        self.assertEqual(UAcceptResult.VALID, status)

        locale, status = LocaleMatcher.acceptLanguageFromHTTP(
            'fr-CH, fr;q=0.9, de;q=0.7', ('en-CA',))
        self.assertEqual(None, locale)
        self.assertEqual(UAcceptResult.FAILED, status)


if __name__ == "__main__":
    if ICU_VERSION >= '65.0':
        main()



# ====================================================================
# Copyright (c) 2020 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *

class TestBidi(TestCase):

    input_text = [
        UnicodeString(b'\xd9\x83\xd8\xa7\xd9\x86 Windows 10 \xd8\xa3\xd9\x88\xd9\x84 \xd9\x86\xd8\xb8\xd8\xa7\xd9\x85 \xd8\xaa\xd8\xb4\xd8\xba\xd9\x8a\xd9\x84 \xd8\xa8\xd8\xaa\xd9\x82\xd9\x86\xd9\x8a\xd8\xa9 4K.', 'utf-8'),
        UnicodeString(b'\xd9\x83\xd8\xa7\xd9\x86\xd8\xaa Microsoft Windows \xd8\xa7\xd9\x84\xd9\x85\xd8\xa7\xd9\x84\xd9\x83 \xd8\xa7\xd9\x84\xd8\xa3\xd9\x88\xd8\xad\xd8\xaf \xd9\x84\xd8\xb9\xd9\x84\xd8\xa7\xd9\x85\xd8\xa9 XP.', 'utf-8')]

    def testDefault(self):

        Locale.setDefault(Locale.getUS())
        layout = Bidi()
        layout.setPara(self.input_text[0])

        self.assertEqual(layout.getParaLevel(), 1)
        self.assertEqual(layout.getDirection(), UBiDiDirection.MIXED)
        self.assertEqual(layout.getLevels(), (1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1))
        self.assertEqual(layout.getVisualRun(0), (39, 1, 1))
        self.assertEqual(layout.getVisualRun(3), (4, 10, 0))

    def testForcedDirection(self):

        Locale.setDefault(Locale.getUS())
        layout = Bidi()
        layout.setPara(self.input_text[0], 0)

        self.assertEqual(layout.getParaLevel(), 0)
        self.assertEqual(layout.getDirection(), UBiDiDirection.MIXED)
        self.assertEqual(layout.getLevels(), (1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 0, 0))
        self.assertEqual(layout.getVisualRun(0), (0, 3, 1))
        self.assertEqual(layout.getVisualRun(3), (15, 22, 1))

    def testLines(self):

        Locale.setDefault(Locale.getUS())
        layout = Bidi()
        layout.setPara(self.input_text[1])

        line_layout_0 = layout.setLine(0, 15)

        self.assertEqual(line_layout_0.getLevels(), (1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1))
        self.assertEqual(line_layout_0.getVisualRun(0), (14, 1, 1))
        self.assertEqual(line_layout_0.getVisualRun(2), (0, 5, 1))

        line_layout_1 = layout.setLine(15, 47)

        self.assertEqual(line_layout_1.getLevels(), (2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1))
        self.assertEqual(line_layout_1.getVisualRun(0), (31, 1, 1))
        self.assertEqual(line_layout_1.getVisualRun(2), (7, 22, 1))

    def testTransform(self):

        if ICU_VERSION >= '58.0':
            Locale.setDefault(Locale.getUS())
            transform = BidiTransform()

            result = transform.transform(
                self.input_text[0],
                UBiDiDirection.LTR, UBiDiOrder.VISUAL,
                UBiDiDirection.RTL, UBiDiOrder.LOGICAL,
                UBiDiMirroring.OFF, Shape.DIGITS_EN2AN)

            self.assertEqual(result, UnicodeString(b'.\xd9\xa4K \xd8\xa9\xd9\x8a\xd9\x86\xd9\x82\xd8\xaa\xd8\xa8 \xd9\x84\xd9\x8a\xd8\xba\xd8\xb4\xd8\xaa \xd9\x85\xd8\xa7\xd8\xb8\xd9\x86 \xd9\x84\xd9\x88\xd8\xa3 \xd9\xa1\xd9\xa0 Windows \xd9\x86\xd8\xa7\xd9\x83', 'utf-8'))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2011-2011 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os, six

from unittest import TestCase, main
from icu import *

try:
    if ICU_VERSION >= '58':
        raise NotImplementedError
    from fontTools.ttLib import TTFont
except ImportError as e:
    sys.stderr.write("\nfontTools package not found, skipping LayoutEngine tests\n")
except NotImplementedError:
    sys.stderr.write("\nLayoutEngine not available in ICU %s" %(ICU_VERSION))
else:
    class TTXLEFont(LEFontInstance):

        def __init__(self, fname, size=12):
            super(TTXLEFont, self).__init__()

            self.ttx = TTFont(fname)
            self.size = size
            self.upem = self.ttx['head'].unitsPerEm
            self.cmap = self.ttx['cmap'].getcmap(3, 1).cmap

        def getFontTable(self, table):
            return self.ttx.getTableData(table)

        def getAscent(self):
            self.ttx['hhea'].ascent * self.size * 1. / self.upem

        def getDescent(self):
            self.ttx['hhea'].descent * self.size * 1. / self.upem

        def getLeading(self):
            self.ttx['hhea'].lineGap * self.size * 1. / self.upem

        def getUnitsPerEm(self):
            return self.upem

        def mapCharToGlyph(self, code):
            return self.ttx.getGlyphID(self.cmap[code])

        def getGlyphAdvance(self, glyph):

            if glyph >= self.ttx['maxp'].numGlyphs:
                return (0., 0.)

            name = self.ttx.getGlyphName(glyph)
            x = self.ttx['hmtx'][name][0] * self.size * 1. / self.upem

            if 'vmtx' in self.ttx:
                y = self.ttx['vmtx'][name][0] * self.size * 1. / self.upem
            else:
                y = 0.

            return (x, y)

        def getGlyphPoint(self, glyph, point):
            return (0., 0.)

        def getXPixelsPerEm(self):
            return self.size

        def getYPixelsPerEm(self):
            return self.size

        def getScaleFactorX(self):
            return 1.

        def getScaleFactorY(self):
            return 1.


try:
    if ICU_VERSION >= '58':
        raise NotImplementedError
    import fontTools
except ImportError:
    pass
except NotImplementedError:
    pass
else:
    class TestLayoutEngine(TestCase):

        def filePath(self, name):

            module = sys.modules[TestLayoutEngine.__module__].__file__
            return os.path.join(os.path.dirname(module), name)

        def setUp(self):

            self.font = TTXLEFont(self.filePath("lohit_hi.ttf"))
            self.layout = LayoutEngine.layoutEngineFactory(self.font,
                                                           ScriptCode.deva,
                                                           LanguageCode.nul)

        def testHello(self):

            self.layout.layoutChars(u"नमस्ते दुनिया")
            self.assertEqual(self.layout.getGlyphCount(), 13)


if __name__ == "__main__":
    try:
        if ICU_VERSION >= '58':
            raise NotImplementedError
        import fontTools
    except ImportError:
        pass
    except NotImplementedError:
        pass
    else:
        main()



# ====================================================================
# Copyright (c) 2014-2014 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *

class TestSpoofChecker(TestCase):
    def setUp(self):
        self.checker = SpoofChecker()

    def testCheck(self):
        checks = self.checker.check("xyz")
        self.assertEqual(checks, 0)

    def testAreConfusable(self):
        checks = self.checker.areConfusable("cxs", u"\u0441\u0445\u0455")
        self.assertEqual(checks, (USpoofChecks.MIXED_SCRIPT_CONFUSABLE |
                                  USpoofChecks.WHOLE_SCRIPT_CONFUSABLE))

        checks = self.checker.areConfusable(
            "A long string that will overflow stack buffers.  A long string that will overflow stack buffers. A long string that will overflow stack buffers.  A long string that will overflow stack buffers. ",
            "A long string that wi11 overflow stack buffers.  A long string that will overflow stack buffers. A long string that wi11 overflow stack buffers.  A long string that will overflow stack buffers. ")
        self.assertEqual(checks, USpoofChecks.SINGLE_SCRIPT_CONFUSABLE)

    def testGetSkeleton(self):
        ML = 0
        SL = USpoofChecks.SINGLE_SCRIPT_CONFUSABLE
        MA = USpoofChecks.ANY_CASE
        SA = USpoofChecks.SINGLE_SCRIPT_CONFUSABLE | USpoofChecks.ANY_CASE

        def checkSkeleton(checks, string, result):
            skeleton = self.checker.getSkeleton(checks, string)
            self.assertEqual(skeleton, result)

        self.maxDiff = None
        checkSkeleton(MA, "I1l0O", "lllOO")

        # A long "identifier" that will overflow implementation stack buffers,
        # forcing heap allocations.

        if ICU_VERSION < '60.1':
            skeleton = u" A long 'identifier' that vvill overflovv irnplernentation stack buffers, forcing heap allocations. A long 'identifier' that vvill overflovv irnplernentation stack buffers, forcing heap allocations. A long 'identifier' that vvill overflovv irnplernentation stack buffers, forcing heap allocations. A long 'identifier' that vvill overflovv irnplernentation stack buffers, forcing heap allocations."
        else:
            skeleton = u" A long 'identifier' that will overflow irnplernentation stack buffers, forcing heap allocations. A long 'identifier' that will overflow irnplernentation stack buffers, forcing heap allocations. A long 'identifier' that will overflow irnplernentation stack buffers, forcing heap allocations. A long 'identifier' that will overflow irnplernentation stack buffers, forcing heap allocations."

        checkSkeleton(SL, u" A 1ong \u02b9identifier' that will overflow implementation stack buffers, forcing heap allocations. A 1ong 'identifier' that will overflow implementation stack buffers, forcing heap allocations. A 1ong 'identifier' that will overflow implementation stack buffers, forcing heap allocations. A 1ong 'identifier' that will overflow implementation stack buffers, forcing heap allocations.",
                      skeleton)

        checkSkeleton(SL, "nochange", "nochange")
        checkSkeleton(MA, "love", "love")
        checkSkeleton(MA, "1ove", "love")   # Digit 1 to letter l
        checkSkeleton(ML, "OOPS", "OOPS")

        if ICU_VERSION < '55.1':
            # Digit 0 unchanged in lower case mode.
            checkSkeleton(ML, "00PS", "00PS")
        else:
            checkSkeleton(ML, "00PS", "OOPS")

        checkSkeleton(MA, "OOPS", "OOPS")
        checkSkeleton(MA, "00PS", "OOPS")   # Digit 0 to letter O in any case mode only
        checkSkeleton(SL, u"\u059c", u"\u0301")
        checkSkeleton(SL, u"\u2A74", u"\u003A\u003A\u003D")
        checkSkeleton(SL, u"\u247E", u"\u0028\u006C\u006C\u0029")  # "(ll)"

        if ICU_VERSION < '55.1':
            checkSkeleton(SL, u"\uFDFB",
                          u"\u062C\u0644\u0020\u062C\u0644\u0627\u0644\u0647")
        else:
            checkSkeleton(SL, u"\uFDFB",
                          u"\u062C\u0644\u0020\u062c\u0644l\u0644o")

        # This mapping exists in the ML and MA tables, does not exist in SL, SA
        # 0C83 ;        0C03 ;
        if ICU_VERSION < '55.1':
            checkSkeleton(SL, u"\u0C83", u"\u0C83")
            checkSkeleton(SA, u"\u0C83", u"\u0C83")
        else:
            checkSkeleton(SL, u"\u0C83", u"\u0983")
            checkSkeleton(SA, u"\u0C83", u"\u0983")

        checkSkeleton(ML, u"\u0C83", u"\u0983")
        checkSkeleton(MA, u"\u0C83", u"\u0983")

        # 0391 ; 0041 ;
        # This mapping exists only in the MA table.
        checkSkeleton(MA, u"\u0391", "A")
        if ICU_VERSION < '55.1':
            checkSkeleton(SA, u"\u0391", u"\u0391")
            checkSkeleton(ML, u"\u0391", u"\u0391")
            checkSkeleton(SL, u"\u0391", u"\u0391")
        else:
            checkSkeleton(SA, u"\u0391", u'A')
            checkSkeleton(ML, u"\u0391", u'A')
            checkSkeleton(SL, u"\u0391", u'A')

        # 13CF ;  0062 ;
        # This mapping exists in the ML and MA tables
        checkSkeleton(ML, u"\u13CF", "b")
        checkSkeleton(MA, u"\u13CF", "b")

        if ICU_VERSION < '55.1':
            checkSkeleton(SL, u"\u13CF", u"\u13CF")
            checkSkeleton(SA, u"\u13CF", u"\u13CF")
        else:
            checkSkeleton(SL, u"\u13CF", u'b')
            checkSkeleton(SA, u"\u13CF", u'b')

        # 0022 ;  0027 0027 ;
        # all tables.
        checkSkeleton(SL, u"\u0022", u"\u0027\u0027")
        checkSkeleton(SA, u"\u0022", u"\u0027\u0027")
        checkSkeleton(ML, u"\u0022", u"\u0027\u0027")
        checkSkeleton(MA, u"\u0022", u"\u0027\u0027")

        # 017F ;  0066 ;
        # This mapping exists in the SA and MA tables
        checkSkeleton(MA, u"\u017F", "f")
        checkSkeleton(SA, u"\u017F", "f")

    def testInvisible(self):

        checks = self.checker.check(u"abcd\u0301ef")
        self.assertEqual(checks, 0)

        checks = self.checker.check(u"abcd\u0301\u0302\u0301ef")
        self.assertEqual(checks, USpoofChecks.INVISIBLE)

        # Two acute accents, one from the composed a with acute accent, \u00e1,
        # and one separate.
        checks = self.checker.check(u"abcd\u00e1\u0301xyz")
        self.assertEqual(checks, USpoofChecks.INVISIBLE)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2009-2015 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os, six

from unittest import TestCase, main
from icu import *


class TestTransliterator(TestCase):

    def testTransliterate(self):

        trans = Transliterator.createInstance('Accents-Any')
        inverse = trans.createInverse()

        string = u'\xe9v\xe9nement'
        if ICU_VERSION < '4.0':
            result = u"e<'>ve<'>nement"
        else:
            result = u"e\u2190'\u2192ve\u2190'\u2192nement"

        self.assertTrue(trans.transliterate(string) == result)
        self.assertTrue(inverse.transliterate(result) == string)

    def testUnicodeString(self):

        trans = Transliterator.createInstance('NumericPinyin-Latin')
        string = UnicodeString("Shang4hai3 zi4lai2shui3 lai2 zi4 hai3 shang4")
        result = u'Sh\xe0ngh\u01cei z\xecl\xe1ishu\u01d0 l\xe1i z\xec h\u01cei sh\xe0ng'

        self.assertTrue(trans.transliterate(six.text_type(string)) == result)
        self.assertTrue(trans.transliterate(string) == result)
        self.assertTrue(string == result)

    def testPythonTransliterator(self):

        class vowelSubst(Transliterator):
            def __init__(_self, char=u'i'):
                super(vowelSubst, _self).__init__("vowel")
                _self.char = char
            def handleTransliterate(_self, text, pos, incremental):
                for i in range(pos.start, pos.limit):
                    if text[i] in u"aeiouüöä":
                        text[i] = _self.char
                pos.start = pos.limit

        trans = vowelSubst()

        # see http://en.wikipedia.org/wiki/Drei_Chinesen_mit_dem_Kontrabass
        string = u"Drei Chinesen mit dem Kontrabass"
        result = u'Drii Chinisin mit dim Kintribiss'
        self.assertTrue(trans.transliterate(string) == result)

        # test registration
        Transliterator.registerInstance(trans)

        regTrans = Transliterator.createInstance("vowel")

        self.assertTrue(regTrans.transliterate(string) == result)

    def testPythonTransliteratorException(self):

        class faultySubst(Transliterator):
            def __init__(_self):
                super(faultySubst, _self).__init__("faulty")
            def handleTransliterate(_self, text, pos, incremental):
                raise ValueError("faulty")

        trans = faultySubst()
        self.assertRaises(ValueError, trans.transliterate, "whatever")


if __name__ == "__main__":
    main()



# ====================================================================
# Copyright (c) 2005-2019 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *


class TestBreakIterator(TestCase):

    def assertIsInstance(self, obj, cls):
        if hasattr(TestCase, 'assertIsInstance'):
            TestCase.assertIsInstance(self, obj, cls)
        else:
            self.assertTrue(isinstance(obj, cls),
                            u'%s is not an instance of %s' % (obj, cls))

    def testCreateInstancePolymorph(self):

        bi = BreakIterator.createWordInstance(Locale.getEnglish())
        self.assertIsInstance(bi, RuleBasedBreakIterator)
        bi = BreakIterator.createLineInstance(Locale.getEnglish())
        self.assertIsInstance(bi, RuleBasedBreakIterator)
        bi = BreakIterator.createCharacterInstance(Locale.getEnglish())
        self.assertIsInstance(bi, RuleBasedBreakIterator)
        bi = BreakIterator.createSentenceInstance(Locale.getEnglish())
        self.assertIsInstance(bi, RuleBasedBreakIterator)
        bi = BreakIterator.createTitleInstance(Locale.getEnglish())
        self.assertIsInstance(bi, RuleBasedBreakIterator)

    def testCreateInstanceFromBinaryRules(self):

        if ICU_VERSION >= '4.8':
            bi = BreakIterator.createWordInstance(Locale.getEnglish())
            self.assertIsInstance(bi, RuleBasedBreakIterator)

            data = bi.getBinaryRules()
            rbi = RuleBasedBreakIterator(data)
            self.assertEqual(data, rbi.getBinaryRules())


if __name__ == "__main__":
    main()



 # ====================================================================
 # Copyright (c) 2004-2024 Open Source Applications Foundation.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
 # to deal in the Software without restriction, including without limitation
 # the rights to use, copy, modify, merge, publish, distribute, sublicense,
 # and/or sell copies of the Software, and to permit persons to whom the
 # Software is furnished to do so, subject to the following conditions: 
 #
 # The above copyright notice and this permission notice shall be included
 # in all copies or substantial portions of the Software. 
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 # OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 # ====================================================================

class ICUError(Exception):
    messages = {}

    def __str__(self):
        return "%s, error code: %d" %(self.args[1], self.args[0])

    def getErrorCode(self):
        return self.args[0]


class InvalidArgsError(Exception):
    pass


from ._icu_ import *
from ._icu_ import VERSION as __version__



# ====================================================================
# Copyright (c) 2016-2019 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os, six

from unittest import TestCase, main
from icu import *

# python2 can be built with 16- or 32-bit unicode chars (UCS2 or UCS4)
# python3 uses 32-bit unicode chars (logically)
def is_unicode_32bit():
    return len(u'\U0001f600') == 1

class TestScript(TestCase):

    def testSurrogatePairs(self):
        pairs = u'a\u0950\u4e2d\U00029efa'
        unicode_32bit = is_unicode_32bit()
        names = [Script.getScript(cp).getShortName() for cp in pairs]
        if unicode_32bit:
            self.assertEqual(['Latn', 'Deva', 'Hani', 'Hani'], names)
        else:
            self.assertTrue(
                (['Latn', 'Deva', 'Hani', 'Zzzz', 'Zzzz'] == names) or
                (['Latn', 'Deva', 'Hani', 'Zyyy', 'Zyyy'] == names))

        pairs = UnicodeString(pairs)
        # iterating UChar, not codepoints
        names = [Script.getScript(c).getShortName() for c in pairs]
        self.assertTrue(
            (['Latn', 'Deva', 'Hani', 'Zzzz', 'Zzzz'] == names) or
            (['Latn', 'Deva', 'Hani', 'Zyyy', 'Zyyy'] == names))

        # iterating codepoints not UChar
        names = [Script.getScript(cp).getShortName()
                 for cp in six.text_type(pairs)]
        if unicode_32bit:
            self.assertEqual(['Latn', 'Deva', 'Hani', 'Hani'], names)
        else:
            self.assertTrue(
                (['Latn', 'Deva', 'Hani', 'Zzzz', 'Zzzz'] == names) or
                (['Latn', 'Deva', 'Hani', 'Zyyy', 'Zyyy'] == names))

        # iterating codepoints, not UChar
        names = [Script.getScript(pairs.char32At(i)).getShortName()
                 for i in range(pairs.countChar32())]
        self.assertEqual(['Latn', 'Deva', 'Hani', 'Hani'], names)

        iterator = StringCharacterIterator(pairs)
        names = []
        while iterator.hasNext():
            desc = Script.getScript(iterator.next32PostInc()).getShortName()
            names.append(desc)
        self.assertEqual(['Latn', 'Deva', 'Hani', 'Hani'], names)

    def testSmileyFace(self):
        char = u'\U0001f600'
        u = UnicodeString(char)
        self.assertEqual(u.countChar32(), 1)

        if sys.version_info >= (3,):
            self.assertEqual(len(char), 1)
            self.assertEqual(str(u), char)
        elif is_unicode_32bit():
            self.assertEqual(len(char), 1)
            self.assertEqual(six.text_type(u), char)
        else:
            self.assertEqual(len(char), 2)
            self.assertEqual(six.text_type(u), char)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2016-2016 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from icu import *


# These tests were lifted from the examples in ``dtptngsample.cpp`` in icu4c

class TestDateTimePatternGenerator(TestCase):

    def setUp(self):
        if ICU_VERSION < '56.0':
            self.skipTest(ICU_VERSION)

        cal = Calendar.createInstance()
        cal.setTimeZone(TimeZone.getGMT())
        cal.set(2016, 4, 10, 0, 30, 0)  # month value is 0-based
        self.date = cal.getTime()
        self.tz = TimeZone.createTimeZone('America/Los_Angeles')

    def testAddPattern(self):
        """Test overwriting a pattern with the same skeleton."""
        locale = Locale.getFrance()
        dtpg = DateTimePatternGenerator.createInstance(locale)

        sdf = SimpleDateFormat(dtpg.getBestPattern('MMMMddHmm'), locale)
        sdf.setTimeZone(self.tz)
        if ICU_VERSION < '68.0' or ICU_VERSION > '71.0':
            self.assertEqual(sdf.format(self.date), u'09 mai à 17:30')
            self.assertEqual(sdf.toPattern(), u"dd MMMM 'à' HH:mm")
        else:
            self.assertEqual(sdf.format(self.date), u'09 mai, 17:30')
            self.assertEqual(sdf.toPattern(), u"dd MMMM, HH:mm")

        dtpg.addPattern("dd'. von' MMMM", True)
        sdf.applyPattern(dtpg.getBestPattern('MMMMddHmm'))
        sdf.setTimeZone(self.tz)
        if ICU_VERSION < '68.0' or ICU_VERSION > '71.0':
            self.assertEqual(sdf.format(self.date), u'09. von mai à 17:30')
            self.assertEqual(sdf.toPattern(), u"dd'. von' MMMM 'à' HH:mm")
        else:
            self.assertEqual(sdf.format(self.date), u'09. von mai, 17:30')
            self.assertEqual(sdf.toPattern(), u"dd'. von' MMMM, HH:mm")

    def testGetBestPattern(self):
        """Test a few different languages and common patterns."""
        skeletons = ('yQQQQ',  # year + full name of quarter, i.e., 4th quarter 1999
                     'yMMMM',  # year + full name of month, i.e., October 1999
                     'MMMMd',  # full name of month + day of the month, i.e., October 25
                     'hhmm',   # 12-hour-cycle format, i.e., 1:32 PM
                     'jjmm')   # preferred hour format for the given locale, i.e., 24-hour-cycle format for fr_FR
        if ICU_VERSION < '72.0':
            locales = (
                (Locale('en_US'),
                 (u'2nd quarter 2016', u'May 2016', u'May 9', u'5:30 PM',
                  u'5:30 PM')),
                (Locale('fr_FR'),
                 (u'2e trimestre 2016', u'mai 2016', u'9 mai', u'5:30 PM',
                  u'17:30')),
                (Locale('zh_CN'),
                 (u'2016年第2季度', u'2016年5月', u'5月9日', u'下午5:30',
                  u'17:30' if ICU_VERSION >= '70.1' else u'下午5:30')),
            )
        else:
            locales = (
                (Locale('en_US'),
                 (u'2nd quarter 2016', u'May 2016', u'May 9', u'5:30\u202fPM',
                  u'5:30\u202fPM')),
                (Locale('fr_FR'),
                 (u'2e trimestre 2016', u'mai 2016', u'9 mai', u'5:30\u202fPM',
                  u'17:30')),
                (Locale('zh_CN'),
                 (u'2016年第2季度', u'2016年5月', u'5月9日', u'下午5:30',
                  u'17:30' if ICU_VERSION >= '70.1' else u'下午5:30')),
            )
            
        for locale, locale_data in locales:
            dtpg = DateTimePatternGenerator.createInstance(locale)
            for index, skeleton in enumerate(skeletons):
                pattern = dtpg.getBestPattern(skeleton)
                sdf = SimpleDateFormat(pattern, locale)
                sdf.setTimeZone(self.tz)
                self.assertEqual(sdf.format(self.date), locale_data[index])

    def testReplaceFieldType(self):
        """Test replacing an element of a pattern."""
        locale = Locale.getFrance()
        dtpg = DateTimePatternGenerator.createInstance(locale)
        sdf = SimpleDateFormat('EEEE d MMMM y HH:mm:ss zzzz', locale)
        sdf.setTimeZone(self.tz)
        pattern = sdf.toPattern()
        self.assertEqual(pattern, u'EEEE d MMMM y HH:mm:ss zzzz')
        if ICU_VERSION < '72.0':
            self.assertEqual(sdf.format(self.date), u'lundi 9 mai 2016 17:30:00 heure d’été du Pacifique')
        else:
            self.assertEqual(sdf.format(self.date), u'lundi 9 mai 2016 17:30:00 heure d’été du Pacifique nord-américain')

        newPattern = dtpg.replaceFieldTypes(pattern, 'vvvv')
        sdf.applyPattern(newPattern)
        sdf.setTimeZone(self.tz)
        self.assertEqual(sdf.toPattern(), u'EEEE d MMMM y HH:mm:ss vvvv')
        self.assertEqual(sdf.format(self.date), u'lundi 9 mai 2016 17:30:00 heure du Pacifique nord-américain')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2019 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================

import sys, os, six

from unittest import TestCase, main
from icu import *

class TestLocale(TestCase):

    def testConstructWithKeywords(self):

        l0 = Locale('fr', 'be', collation='phonebook', currency='euro')
        l1 = Locale('fr', 'be', '', 'collation=phonebook;currency=euro')

        self.assertEqual('fr_BE@collation=phonebook;currency=euro', str(l0))
        self.assertEqual('fr_BE@collation=phonebook;currency=euro', str(l1))
        self.assertEqual(l0, l1)

    def testCompare(self):

        af = Locale('af')
        mt = Locale('mt')
        zu = Locale('zu')

        self.assertLess(af, zu)
        self.assertFalse(mt < mt)
        self.assertFalse(zu < af)

        self.assertLessEqual(af, zu)
        self.assertLessEqual(mt, mt)
        self.assertFalse(zu <= af)

        self.assertEqual(mt, mt)
        self.assertFalse(af == zu)

        self.assertNotEqual(af, zu)
        self.assertFalse(mt != mt)

        self.assertGreater(zu, af)
        self.assertFalse(mt > mt)
        self.assertFalse(af > zu)

        self.assertGreaterEqual(zu, af)
        self.assertGreaterEqual(mt, mt)
        self.assertFalse(af >= zu)


if __name__ == "__main__":
    main()




import os, sys

try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

from distutils.spawn import find_executable

VERSION = '2.16.1'
ICU_MAX_MAJOR_VERSION = '78'  # max supported major version of ICU

try:
    from subprocess import check_output as subprocess_check_output

    def check_output(popenargs):
        print("(running '%s')" %(' '.join(popenargs)))
        return subprocess_check_output(popenargs)

except ImportError:
    from subprocess import Popen, PIPE

    def check_output(*popenargs):
        print("(running '%s')" %(' '.join(*popenargs)))
        process = Popen(stdout=PIPE, *popenargs)
        output, ignore = process.communicate()
        retcode = process.poll()
        if retcode:
            raise RuntimeError((retcode, popenargs[0], output))
        return output


def configure_with_icu_config(flags, config_args, label):
    try:
        output = check_output(('icu-config',) + config_args).strip()
        if sys.version_info >= (3,):
            output = str(output, 'ascii')
        flags.extend(output.split())
        if output:
            print('Adding %s="%s" from %s' % (label, output,
                                              find_executable('icu-config')))
    except:
        print('Could not configure %s with icu-config' %(label))
        raise


def configure_with_pkg_config(flags, config_args, label):
    try:
        output = check_output(('pkg-config',) + config_args + ('icu-i18n',)).strip()
        if sys.version_info >= (3,):
            output = str(output, 'ascii')
        flags.extend(output.split())
        if output:
            print('Adding %s="%s" from %s' % (label, output,
                                              find_executable('pkg-config')))
    except:
        print('Could not configure %s with pkg-config' %(label))
        raise


platform = sys.platform
if platform.startswith(('linux', 'gnu')):
    platform = 'linux'
elif platform.startswith('freebsd'):
    platform = 'freebsd'


CONFIGURE_WITH_ICU_CONFIG = {
    'darwin': False,
    'linux': True,
    'freebsd': False, # not tested
    'win32': False,   # no icu-config
    'sunos5': False,  # not tested
    'cygwin': False,  # not tested
}

CONFIGURE_WITH_PKG_CONFIG = {
    'darwin': False,  # no pkg-config ?
    'linux': True,
    'freebsd': False, # not tested
    'win32': False,   # no pkg-config ?
    'sunos5': False,  # not tested
    'cygwin': False,  # not tested
}


try:
    ICU_VERSION = os.environ['ICU_VERSION']
except:
    try:
        ICU_VERSION = check_output(('icu-config', '--version')).strip()
        CONFIGURE_WITH_ICU_CONFIG[platform] = True
    except:
        try:
            ICU_VERSION = check_output(('pkg-config', '--modversion', 'icu-i18n')).strip()
            CONFIGURE_WITH_PKG_CONFIG[platform] = True
        except:
            raise RuntimeError('''
Please install pkg-config on your system or set the ICU_VERSION environment
variable to the version of ICU you have installed.
        ''')
    if sys.version_info >= (3,):
        ICU_VERSION = str(ICU_VERSION, 'ascii')

print('''
Building pyicu %s for ICU %s (max ICU major version supported: %s)
''' %(VERSION, ICU_VERSION, ICU_MAX_MAJOR_VERSION))

INCLUDES = {
    'darwin': [],
    'linux': [],
    'freebsd': ['/usr/local/include'],
    'win32': ['c:/icu/include'],
    'sunos5': [],
    'cygwin': [],
}

if sys.platform == 'win32' and sys.version_info < (3,9):
    ver_flag = '/D%s=\\"%s\\"'
else:
    ver_flag = '-D%s="%s"'

VER_FLAGS = [ver_flag %('PYICU_VER', VERSION),
             ver_flag %('PYICU_ICU_MAX_VER', ICU_MAX_MAJOR_VERSION)]

# added to CFLAGS when setup is invoked with --pedantic
PEDANTIC_FLAGS = {
    'darwin': ['-pedantic'],
    'linux': ['-pedantic', '-Wno-variadic-macros'],
    'freebsd': ['-pedantic'],
    'win32': [],
    'sunos5': [],
    'cygwin': ['-pedantic'],
}

CFLAGS = {
    'darwin': ['-std=c++17'],
    'linux': ['-std=c++17'],
    'freebsd': ['-std=c++17'],
    'win32': ['/Zc:wchar_t', '/EHsc'],
    'sunos5': ['-std=c++17'],
    'cygwin': ['-D_GNU_SOURCE=1', '-std=c++17'],
}

# added to CFLAGS when setup is invoked with --debug
DEBUG_CFLAGS = {
    'darwin': ['-O0', '-g', '-DDEBUG'],
    'linux': ['-O0', '-g', '-DDEBUG'],
    'freebsd': ['-O0', '-g', '-DDEBUG'],
    'win32': ['/Od', '/DDEBUG'],
    'sunos5': ['-DDEBUG'],
    'cygwin': ['-Og', '-g', '-DDEBUG'],
}

LFLAGS = {
    'darwin': [],
    'linux': [],
    'freebsd': ['-L/usr/local/lib'],
    'win32': ['/LIBPATH:c:/icu/lib'],
    'sunos5': [],
    'cygwin': [],
}

LIBRARIES = {
    'darwin': [],
    'linux': [],
    'freebsd': ['icui18n', 'icuuc', 'icudata'],
    'win32': ['icuin', 'icuuc', 'icudt'],
    'sunos5': ['icui18n', 'icuuc', 'icudata'],
    'cygwin': ['icui18n', 'icuuc', 'icudata'],
}

if 'PYICU_INCLUDES' in os.environ:
    _includes = os.environ['PYICU_INCLUDES'].split(os.pathsep)
else:
    _includes = INCLUDES[platform]

if 'PYICU_CFLAGS' in os.environ:
    _cflags = os.environ['PYICU_CFLAGS'].split(os.pathsep)
else:
    _cflags = CFLAGS[platform]
    try:
        if CONFIGURE_WITH_ICU_CONFIG[platform]:
            try:
                configure_with_icu_config(
                    _cflags, ('--cxxflags', '--cppflags'), 'CFLAGS')
            except:
                if CONFIGURE_WITH_PKG_CONFIG[platform]:
                    configure_with_pkg_config(_cflags, ('--cflags',), 'CFLAGS')
                else:
                    raise
        elif CONFIGURE_WITH_PKG_CONFIG[platform]:
            configure_with_pkg_config(_cflags, ('--cflags',), 'CFLAGS')
    except:
        if not _cflags:
            raise RuntimeError('''
Please install pkg-config on your system or set the PYICU_CFLAGS environment 
variable to the flags required by the C++ compiler to find the header files
for ICU, and possibly -std=c++11 if using ICU version >= 60 or -std=c++17 if using ICU version >= 75
            ''')

if '--debug' in sys.argv:
    sys.argv.remove('--debug')
    if 'PYICU_DEBUG_CFLAGS' in os.environ:
        _cflags += os.environ['PYICU_DEBUG_CFLAGS'].split(os.pathsep)
    else:
        _cflags += DEBUG_CFLAGS[platform]

if '--pedantic' in sys.argv:
    sys.argv.remove('--pedantic')
    _cflags += PEDANTIC_FLAGS[platform]

_cflags += VER_FLAGS

if 'PYICU_LFLAGS' in os.environ:
    _lflags = os.environ['PYICU_LFLAGS'].split(os.pathsep)
else:
    _lflags = LFLAGS[platform]
    try:
        if CONFIGURE_WITH_ICU_CONFIG[platform]:
            try:
                configure_with_icu_config(_lflags, ('--ldflags',), 'LFLAGS')
            except:
                if CONFIGURE_WITH_PKG_CONFIG[platform]:
                    configure_with_pkg_config(_lflags, ('--libs',), 'LFLAGS')
                else:
                    raise
        elif CONFIGURE_WITH_PKG_CONFIG[platform]:
            configure_with_pkg_config(_lflags, ('--libs',), 'LFLAGS')
    except:
        if not _lflags:
            raise RuntimeError('''
Please install pkg-config on your system or set the PYICU_LFLAGS environment
variable to the flags required by the linker to find the libraries for ICU
            ''')
      

if 'PYICU_LIBRARIES' in os.environ:
    _libraries = os.environ['PYICU_LIBRARIES'].split(os.pathsep)
elif ICU_VERSION < '58':
    _libraries = LIBRARIES[platform][:] + ['icule']
else:
    _libraries = LIBRARIES[platform]

if sys.version_info < (2, 4):
    def sorted(iterable, *args, **kwargs):
        iterable = list(iterable)
        iterable.sort(*args, **kwargs)
        return iterable


setup(name="pyicu",
      description='Python extension wrapping the ICU C++ API',
      long_description=open('README.md').read(),
      long_description_content_type="text/markdown",
      version=VERSION,
      test_suite="test",
      url='https://gitlab.pyicu.org/main/pyicu',
      author='Andi Vajda',
      author_email='vajda@pyicu.org',
      license='MIT',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved',
          'Operating System :: OS Independent',
          'Programming Language :: C++',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: PyPy',
          'Topic :: Software Development :: Localization',
          'Topic :: Software Development :: Internationalization'],
      ext_modules=[Extension('icu._icu_',
                             [filename for filename in sorted(os.listdir(os.curdir))
                              if filename.endswith('.cpp')],
                             include_dirs=_includes,
                             extra_compile_args=_cflags,
                             extra_link_args=_lflags,
                             libraries=_libraries)],
      package_dir={"": "py"},
      packages=['icu'],
      tests_require=['pytest', 'six'])



# ====================================================================
# Copyright (c) 2004-2010 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from icu import *


def processCollator(options):

    if options.rules is not None:
        rules = unicode(options.rules, 'unicode-escape')
        collator = RuleBasedCollator(rules, Collator.TERTIARY,
                                     UCollAttributeValue.OFF)
    else:
        collator = Collator.createInstance(Locale(options.locale))

    if options.norm:
        collator.setAttribute(UCollAttribute.NORMALIZATION_MODE,
                              UCollAttributeValue.ON)
    if options.french:
        collator.setAttribute(UCollAttribute.FRENCH_COLLATION,
                              UCollAttributeValue.ON)
    if options.lower:
        collator.setAttribute(UCollAttribute.CASE_FIRST,
                              UCollAttributeValue.LOWER_FIRST)
    if options.upper:
        collator.setAttribute(UCollAttribute.CASE_FIRST,
                              UCollAttributeValue.UPPER_FIRST)
    if options.case:
        collator.setAttribute(UCollAttribute.CASE_LEVEL,
                              UCollAttributeValue.ON)
    if options.shifted:
        collator.setAttribute(UCollAttribute.ALTERNATE_HANDLING,
                              UCollAttributeValue.SHIFTED)
    if options.level == 1:
        collator.setAttribute(UCollAttribute.STRENGTH,
                              UCollAttributeValue.PRIMARY)
    elif options.level == 2:
        collator.setAttribute(UCollAttribute.STRENGTH,
                              UCollAttributeValue.SECONDARY)
    elif options.level == 3:
        collator.setAttribute(UCollAttribute.STRENGTH,
                              UCollAttributeValue.TERTIARY)
    elif options.level == 4:
        collator.setAttribute(UCollAttribute.STRENGTH,
                              UCollAttributeValue.QUATERNARY)
    elif options.level == 5:
        collator.setAttribute(UCollAttribute.STRENGTH,
                              UCollAttributeValue.IDENTICAL)
    else:
        raise ValueError, "--level must be between 1 and 5"

    return collator


def processStringSearch(options, collator):

    source = unicode(options.source, "unicode-escape")
    pattern = unicode(options.pattern, "unicode-escape")

    search = StringSearch(pattern, source, collator)
    if options.overlap:
        search.setAttribute(USearchAttribute.OVERLAP,
                            USearchAttributeValue.ON)
    if options.canonical:
        search.setAttribute(USearchAttribute.CANONICAL_MATCH,
                            USearchAttributeValue.ON)
    return search


def findPattern(search):

    found = False
    for offset in search:
        print("Pattern found at offset", offset, "size", search.getMatchedLength())
        found = True

    if not found:
        print("Pattern not found in source")


if __name__ == "__main__":
    from optparse import OptionParser

    parser = OptionParser(usage='''
strsrch [options*] --source source_string --pattern pattern_string

Example:
python strsrch.py --rules \\u0026b\\u003ca 
                  --source a\\u0020b\\u0020bc --pattern b
The format \\uXXXX is supported for the rules and comparison strings''')

    parser.add_option('-l', "--locale", dest='locale', default='en_US',
                      help="ICU locale to use.  Default is en_US")
    parser.add_option('-r', "--rules", dest='rules', default=None,
                      help="Collation rules file (overrides locale)")
    parser.add_option('-f', "--french", dest='french', default=False,
                      action="store_true", help="French accent ordering")
    parser.add_option('-n', "--norm", dest='norm', default=False,
                      action="store_true", help="Normalizing mode on")
    parser.add_option('-s', "--shifted", dest='shifted', default=False,
                      action="store_true", help="Shifted mode")
    parser.add_option('-L', "--lower", dest='lower', default=False,
                      action="store_true", help="Lower case first")
    parser.add_option('-U', "--upper", dest='upper', default=False,
                      action="store_true", help="Upper case first")
    parser.add_option('-C', "--case", dest='case', default=False,
                      action="store_true", help="Enable separate case level")
    parser.add_option('-S', "--sort", dest='level', default='1',
                      help="Sort level, 1 to 5, for Primary, Secondary, Tertiary, Quaternary, Identical")
    parser.add_option('-u', "--source", dest='source',
                      default="International Components for Unicode",
                      help="Source string")
    parser.add_option('-p', "--pattern", dest='pattern',
                      default="Unicode",
                      help="Pattern string to look for in source")
    parser.add_option('-o', "--overlap", dest='overlap', default=False,
                      action="store_true", help="Enable searching to be done on overlapping patterns")
    parser.add_option('-c', "--canonical", dest='canonical', default=False,
                      action="store_true", help="Enable searching to be done matching canonical equivalent patterns")

    options, args = parser.parse_args()
    options.level = int(options.level)

    collator = processCollator(options)
    search = processStringSearch(options, collator)

    print("Finding pattern", options.pattern, "in source", options.source)

    findPattern(search)
    del search
    del collator



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2018 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import sys, os

from unittest import TestCase, main
from icu import *

class TestNumberFormatter(TestCase):

    def setUp(self):
        if ICU_VERSION < '60.0':
            self.skipTest(ICU_VERSION)

    def testBasic(self):

        text = NumberFormatter.withLocale(Locale.getUS()).formatInt(1234)
        self.assertEqual(text, u'1,234')

        text = LocalizedNumberFormatter(Locale.getUS()).formatInt(1234)
        self.assertEqual(text, u'1,234')

    def testFormattedNumber(self):

        if ICU_VERSION >= '64.0':
            value = NumberFormatter.withLocale(Locale.getUS()) \
                .formatIntToValue(1234)
            self.assertEqual(str(value), u'1,234')
            self.assertEqual(repr(value), u'<FormattedNumber: 1,234>')

            self.assertEqual([(x.getStart(), x.getLimit()) for x in value],
                             [(1, 2), (0, 5)])

    def testFancy(self):

        if ICU_VERSION < '64.0':
            text = NumberFormatter.with_() \
                .notation(Notation.compactShort()) \
                .unit(CurrencyUnit('EUR')) \
                .rounding(Rounder.maxDigits(2)) \
                .locale(Locale.getFrance()) \
                .formatInt(1234)
            self.assertEqual(text, u'1,2\xa0k\xa0€')

        if ICU_VERSION >= '62.0':
            text = NumberFormatter.with_() \
                .notation(Notation.compactShort()) \
                .unit(CurrencyUnit('EUR')) \
                .precision(Precision.maxSignificantDigits(2)) \
                .locale(Locale.getFrance()) \
                .formatInt(1234)
            self.assertEqual(text, u'1,2\xa0k\xa0€')

    def testUnit(self):

        formatter = UnlocalizedNumberFormatter() \
            .sign(UNumberSignDisplay.ALWAYS) \
            .unit(MeasureUnit.createMeter()) \
            .unitWidth(UNumberUnitWidth.FULL_NAME)

        text = formatter.locale(Locale.getUS()).formatInt(1234)
        self.assertEqual(text, u'+1,234 meters')

        text = formatter.locale(Locale.getFrance()).formatInt(1234)
        if ICU_VERSION >= '63.0':
            self.assertEqual(text, u'+1\u202f234\xa0mètres')
        else:
            self.assertEqual(text, u'+1\xa0234 mètres')

        if ICU_VERSION >= '61.0':
            formatter = UnlocalizedNumberFormatter() \
                .sign(UNumberSignDisplay.ALWAYS) \
                .unit(MeasureUnit.createMeter()) \
                .perUnit(MeasureUnit.createSecond()) \
                .unitWidth(UNumberUnitWidth.FULL_NAME)

            text = formatter.locale(Locale.getUS()).formatInt(1234)
            self.assertEqual(text, u'+1,234 meters per second')

    def testFromDecimalFormat(self):

        if ICU_VERSION >= '68.0':
            text = NumberFormat.createScientificInstance(Locale("de", "at")) \
                .toNumberFormatter() \
                .formatInt(1234)
            self.assertEqual(text, u'1,234E3')


    def testUsage(self):

        if ICU_VERSION >= '68.0':
            formatter = NumberFormatter.with_().usage("person-height").unit(MeasureUnit.createMeter()).locale(Locale("en-US"))

            self.assertEqual(formatter.formatDouble(0.25), '9.8 in')
            self.assertEqual(formatter.formatDouble(1.5), '4 ft, 11 in')
            unit = formatter.formatDoubleToValue(1.5).getOutputUnit()
            self.assertEqual(unit.getComplexity(), UMeasureUnitComplexity.MIXED)
            self.assertEqual(unit.getIdentifier(), "foot-and-inch")


class TestNumberRangeFormatter(TestCase):

    def setUp(self):
        if ICU_VERSION < '63.0':
            self.skipTest(ICU_VERSION)

    if ICU_VERSION < '76.0':
        def testBasic(self):

            text = NumberRangeFormatter.withLocale(Locale.getItaly()) \
                .formatIntRange(1234, 5678)
            self.assertEqual(text, u'1.234-5.678')

            text = LocalizedNumberRangeFormatter(Locale.getItaly()) \
                .formatIntRange(1234, 5678)
            self.assertEqual(text, u'1.234-5.678')

    if ICU_VERSION >= '76.0':
        def testBasic(self):

            text = NumberRangeFormatter.withLocale(Locale.getItaly()) \
                .formatIntRange(1234, 5678)
            self.assertEqual(text, u'1234-5678')

            text = LocalizedNumberRangeFormatter(Locale.getItaly()) \
                .formatIntRange(1234, 5678)
            self.assertEqual(text, u'1234-5678')

    def testFormattedNumberRange(self):

        if ICU_VERSION >= '64.0':
            p2_formatter = NumberFormatter.with_() \
                .precision(Precision.maxSignificantDigits(2))
            p3_formatter = NumberFormatter.with_() \
                .precision(Precision.maxSignificantDigits(3))
            it_formatter = NumberRangeFormatter.withLocale(Locale.getItaly())

            text = it_formatter.numberFormatterBoth(p2_formatter) \
                .formatDoubleRange(1/3.0, 1/4.0)
            self.assertEqual(text, u'0,33-0,25')

            text = it_formatter \
                .numberFormatterFirst(p3_formatter) \
                .numberFormatterSecond(p2_formatter) \
                .formatFormattableRange(Formattable(1/3.0), Formattable(1/4.0))
            self.assertEqual(text, u'0,333-0,25')

            value = it_formatter \
                .numberFormatterFirst(p3_formatter) \
                .numberFormatterSecond(p2_formatter) \
                .formatDoubleRangeToValue(1/3.0, 1/4.0)
            self.assertEqual(str(value), u'0,333-0,25')
            self.assertEqual(repr(value), u'<FormattedNumberRange: 0,333-0,25>')

            if ICU_VERSION < '69.0':
                self.assertEqual(value.getFirstDecimal(), u'3.33E-1')
                self.assertEqual(value.getSecondDecimal(), u'2.5E-1')
                self.assertEqual(
                    [(x.getStart(), x.getLimit()) for x in value],
                    [(0, 1), (1, 2), (2, 5), (6, 7), (7, 8), (8, 10)])
            else:
                self.assertEqual(
                    [(x.getStart(), x.getLimit()) for x in value],
                    [(0, 5), (0, 1), (1, 2), (2, 5), (6, 10), (6, 7), (7, 8),
                     (8, 10)])

            if ICU_VERSION >= '68.0':
                self.assertEqual(value.getDecimalNumbers(), (b'0.333', b'0.25'))


class TestCurrencyUnit(TestCase):

    def testGetName(self):

        cu = CurrencyUnit("USD")
        self.assertEqual(cu.getName(Locale.getUS()), u'$')
        self.assertEqual(
            cu.getName(Locale.getUS(), UCurrNameStyle.LONG_NAME),
            u'US Dollar')
        self.assertEqual(
            cu.getName(Locale.getItaly(), UCurrNameStyle.LONG_NAME),
            u'dollaro statunitense')
        if ICU_VERSION >= '70.0':
            self.assertEqual(cu.getName(Locale.getItaly(),
                                        UCurrNameStyle.FORMAL_SYMBOL_NAME),
                             u'USD')
            self.assertEqual(cu.getName(Locale.getFrance(),
                                        UCurrNameStyle.FORMAL_SYMBOL_NAME),
                             u'$US')
                

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ====================================================================
# Copyright (c) 2019 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================

import sys, os, six

from unittest import TestCase, main
from icu import *

class TestUCharsTrie(TestCase):

    def setUp(self):
        if ICU_VERSION < '4.8':
            self.skipTest(ICU_VERSION)

    def testBuildAndIterate(self):

        mappings = { 'ab': 3, 'abc': 6, 'abcd': 2, 'abcef': 11,
                     'abcp': 88, 'abcqr': 20 }

        builder = UCharsTrie.Builder()
        for key, value in mappings.items():
            builder.add(key, value)
        trie = builder.build(UStringTrieBuildOption.FAST)

        for key, value in trie:
            self.assertEqual(mappings[key], value)

    def testAccess(self):

        mappings = { 'ab': 3, 'abc': 6, 'abcd': 2, 'abcef': 11,
                     'abcp': 88, 'abcqr': 20 }

        builder = UCharsTrie.Builder()
        for key, value in mappings.items():
            builder.add(key, value)
        trie = builder.build(UStringTrieBuildOption.FAST)

        self.assertEqual((3, 6), (trie.next('abc'), trie.getValue()))
        state = trie.saveState()

        self.assertEqual(u'depq', trie.getNextUChars())

        self.assertEqual((2, 2), (trie.next('d'), trie.getValue()))
        self.assertEqual((0, None), (trie.next('e'), trie.getValue()))

        trie.resetToState(state)
        self.assertEqual((2, 11), (trie.next('ef'), trie.getValue()))

        trie.resetToState(state)
        self.assertEqual((2, 88), (trie.next('p'), trie.getValue()))


if __name__ == "__main__":
    if ICU_VERSION >= '4.8':
        main()
